@@ -34,15 +34,21 @@
 
 use std;
 use std::io::Read;
+use std::collections::HashMap;
 
 // Symbols we need from our rust api.
 use MediaContext;
 use TrackType;
 use read_mp4;
 use Error;
-use media_time_to_ms;
-use track_time_to_ms;
 use SampleEntry;
+use AudioCodecSpecific;
+use VideoCodecSpecific;
+use MediaTimeScale;
+use MediaScaledTime;
+use TrackTimeScale;
+use TrackScaledTime;
+use serialize_opus_header;
 
 // rusty-cheddar's C enum generation doesn't namespace enum members by
 // prefixing them, so we're forced to do it in our member names until
@@ -60,8 +66,7 @@ pub enum mp4parse_error {
     MP4PARSE_ERROR_INVALID = 2,
     MP4PARSE_ERROR_UNSUPPORTED = 3,
     MP4PARSE_ERROR_EOF = 4,
-    MP4PARSE_ERROR_ASSERT = 5,
-    MP4PARSE_ERROR_IO = 6,
+    MP4PARSE_ERROR_IO = 5,
 }
 
 #[repr(C)]
@@ -71,15 +76,42 @@ pub enum mp4parse_track_type {
     MP4PARSE_TRACK_TYPE_AUDIO = 1,
 }
 
+#[repr(C)]
+#[derive(PartialEq, Debug)]
+pub enum mp4parse_codec {
+    MP4PARSE_CODEC_UNKNOWN,
+    MP4PARSE_CODEC_AAC,
+    MP4PARSE_CODEC_OPUS,
+    MP4PARSE_CODEC_AVC,
+    MP4PARSE_CODEC_VP9,
+}
+
 #[repr(C)]
 pub struct mp4parse_track_info {
     pub track_type: mp4parse_track_type,
+    pub codec: mp4parse_codec,
     pub track_id: u32,
     pub duration: u64,
     pub media_time: i64, // wants to be u64? understand how elst adjustment works
     // TODO(kinetik): include crypto guff
 }
 
+#[repr(C)]
+pub struct mp4parse_codec_specific_config {
+    pub length: u32,
+    pub data: *const u8,
+}
+
+impl Default for mp4parse_codec_specific_config {
+    fn default() -> Self {
+        mp4parse_codec_specific_config {
+            length: 0,
+            data: std::ptr::null_mut(),
+        }
+    }
+}
+
+#[derive(Default)]
 #[repr(C)]
 pub struct mp4parse_track_audio_info {
     pub channels: u16,
@@ -88,8 +120,7 @@ pub struct mp4parse_track_audio_info {
     // TODO(kinetik):
     // int32_t profile;
     // int32_t extended_profile; // check types
-    // extra_data
-    // codec_specific_config
+    codec_specific_config: mp4parse_codec_specific_config,
 }
 
 #[repr(C)]
@@ -109,6 +140,7 @@ struct Wrap {
     context: MediaContext,
     io: mp4parse_io,
     poisoned: bool,
+    opus_header: HashMap<u32, Vec<u8>>,
 }
 
 #[repr(C)]
@@ -135,6 +167,10 @@ impl mp4parse_parser {
     fn set_poisoned(&mut self, poisoned: bool) {
         self.0.poisoned = poisoned;
     }
+
+    fn opus_header_mut(&mut self) -> &mut HashMap<u32, Vec<u8>> {
+        &mut self.0.opus_header
+    }
 }
 
 #[repr(C)]
@@ -144,12 +180,6 @@ pub struct mp4parse_io {
     pub userdata: *mut std::os::raw::c_void,
 }
 
-// Required for the panic-catching thread in mp4parse_read because raw
-// pointers don't impl Send by default.  This is *only* safe because we wait
-// on the panic-catching thread to complete before returning from
-// mp4parse_read and there's no concurrent access.
-unsafe impl Send for mp4parse_io {}
-
 impl Read for mp4parse_io {
     fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
         if buf.len() > isize::max_value() as usize {
@@ -180,7 +210,12 @@ pub unsafe extern fn mp4parse_new(io: *const mp4parse_io) -> *mut mp4parse_parse
     if ((*io).read as *mut std::os::raw::c_void).is_null() {
         return std::ptr::null_mut();
     }
-    let parser = Box::new(mp4parse_parser(Wrap { context: MediaContext::new(), io: (*io).clone(), poisoned: false }));
+    let parser = Box::new(mp4parse_parser(Wrap {
+        context: MediaContext::new(),
+        io: (*io).clone(),
+        poisoned: false,
+        opus_header: HashMap::new(),
+    }));
     Box::into_raw(parser)
 }
 
@@ -202,50 +237,64 @@ pub unsafe extern fn mp4parse_read(parser: *mut mp4parse_parser) -> mp4parse_err
     let mut context = (*parser).context_mut();
     let mut io = (*parser).io_mut();
 
-    let r = if cfg!(not(feature = "fuzz")) {
-        // Parse in a subthread to catch any panics.
-        let task = std::thread::spawn(move || read_mp4(io, context));
-        // The task's JoinHandle will return an error result if the thread
-        // panicked, and will wrap the closure's return'd result in an
-        // Ok(..) otherwise, meaning we could see Ok(Err(Error::..))
-        // here. So map thread failures back to an
-        // mp4parse::Error::AssertCaught.
-        task.join().unwrap_or_else(|_| Err(Error::AssertCaught))
-    } else {
-        read_mp4(io, context)
-    };
-    (*parser).set_poisoned(r.is_err());
+    let r = read_mp4(io, context);
     match r {
         Ok(_) => MP4PARSE_OK,
-        Err(Error::NoMoov) | Err(Error::InvalidData(_)) => MP4PARSE_ERROR_INVALID,
+        Err(Error::NoMoov) | Err(Error::InvalidData(_)) => {
+            // Block further calls. We've probable lost sync.
+            (*parser).set_poisoned(true);
+            MP4PARSE_ERROR_INVALID
+        }
         Err(Error::Unsupported(_)) => MP4PARSE_ERROR_UNSUPPORTED,
         Err(Error::UnexpectedEOF) => MP4PARSE_ERROR_EOF,
-        Err(Error::AssertCaught) => MP4PARSE_ERROR_ASSERT,
-        Err(Error::Io(_)) => MP4PARSE_ERROR_IO,
+        Err(Error::Io(_)) => {
+            // Block further calls after a read failure.
+            // Getting std::io::ErrorKind::UnexpectedEof is normal
+            // but our From trait implementation should have converted
+            // those to our Error::UnexpectedEOF variant.
+            (*parser).set_poisoned(true);
+            MP4PARSE_ERROR_IO
+        }
     }
 }
 
 /// Return the number of tracks parsed by previous `mp4parse_read()` call.
 #[no_mangle]
-pub unsafe extern fn mp4parse_get_track_count(parser: *const mp4parse_parser) -> u32 {
-    // Validate argument from C.
-    assert!(!parser.is_null() && !(*parser).poisoned());
+pub unsafe extern fn mp4parse_get_track_count(parser: *const mp4parse_parser, count: *mut u32) -> mp4parse_error {
+    // Validate arguments from C.
+    if parser.is_null() || count.is_null() || (*parser).poisoned() {
+        return MP4PARSE_ERROR_BADARG;
+    }
     let context = (*parser).context();
 
     // Make sure the track count fits in a u32.
-    assert!(context.tracks.len() < u32::max_value() as usize);
-    context.tracks.len() as u32
+    if context.tracks.len() > u32::max_value() as usize {
+        return MP4PARSE_ERROR_INVALID;
+    }
+    *count = context.tracks.len() as u32;
+    MP4PARSE_OK
+}
+
+fn media_time_to_ms(time: MediaScaledTime, scale: MediaTimeScale) -> u64 {
+    assert!(scale.0 != 0);
+    time.0 * 1000000 / scale.0
+}
+
+fn track_time_to_ms(time: TrackScaledTime, scale: TrackTimeScale) -> u64 {
+    assert!(time.1 == scale.1);
+    assert!(scale.0 != 0);
+    time.0 * 1000000 / scale.0
 }
 
 /// Fill the supplied `mp4parse_track_info` with metadata for `track`.
 #[no_mangle]
-pub unsafe extern fn mp4parse_get_track_info(parser: *mut mp4parse_parser, track: u32, info: *mut mp4parse_track_info) -> mp4parse_error {
+pub unsafe extern fn mp4parse_get_track_info(parser: *mut mp4parse_parser, track_index: u32, info: *mut mp4parse_track_info) -> mp4parse_error {
     if parser.is_null() || info.is_null() || (*parser).poisoned() {
         return MP4PARSE_ERROR_BADARG;
     }
 
     let context = (*parser).context_mut();
-    let track_index: usize = track as usize;
+    let track_index: usize = track_index as usize;
     let info: &mut mp4parse_track_info = &mut *info;
 
     if track_index >= context.tracks.len() {
@@ -258,40 +307,62 @@ pub unsafe extern fn mp4parse_get_track_info(parser: *mut mp4parse_parser, track
         TrackType::Unknown => return MP4PARSE_ERROR_UNSUPPORTED,
     };
 
-    // Maybe context & track should just have a single simple is_valid() instead?
-    if context.timescale.is_none() ||
-       context.tracks[track_index].timescale.is_none() ||
-       context.tracks[track_index].duration.is_none() ||
-       context.tracks[track_index].track_id.is_none() {
-        return MP4PARSE_ERROR_INVALID;
-    }
+    info.codec = match context.tracks[track_index].data {
+        Some(SampleEntry::Audio(ref audio)) => match audio.codec_specific {
+            AudioCodecSpecific::OpusSpecificBox(_) =>
+                mp4parse_codec::MP4PARSE_CODEC_OPUS,
+            AudioCodecSpecific::ES_Descriptor(_) =>
+                mp4parse_codec::MP4PARSE_CODEC_AAC,
+        },
+        Some(SampleEntry::Video(ref video)) => match video.codec_specific {
+            VideoCodecSpecific::VPxConfig(_) =>
+                mp4parse_codec::MP4PARSE_CODEC_VP9,
+            VideoCodecSpecific::AVCConfig(_) =>
+                mp4parse_codec::MP4PARSE_CODEC_AVC,
+        },
+        _ => mp4parse_codec::MP4PARSE_CODEC_UNKNOWN,
+    };
 
     let track = &context.tracks[track_index];
-    info.media_time = track.media_time.map_or(0, |media_time| {
-        track_time_to_ms(media_time, track.timescale.unwrap()) as i64
-    }) - track.empty_duration.map_or(0, |empty_duration| {
-        media_time_to_ms(empty_duration, context.timescale.unwrap()) as i64
-    });
-    info.duration = track_time_to_ms(track.duration.unwrap(), track.timescale.unwrap());
-    info.track_id = track.track_id.unwrap();
+
+    if let (Some(track_timescale),
+            Some(context_timescale),
+            Some(track_duration)) = (track.timescale,
+                                     context.timescale,
+                                     track.duration) {
+        info.media_time = track.media_time.map_or(0, |media_time| {
+            track_time_to_ms(media_time, track_timescale) as i64
+        }) - track.empty_duration.map_or(0, |empty_duration| {
+            media_time_to_ms(empty_duration, context_timescale) as i64
+        });
+
+        info.duration = track_time_to_ms(track_duration, track_timescale);
+    } else {
+        return MP4PARSE_ERROR_INVALID
+    }
+
+    info.track_id = match track.track_id {
+        Some(track_id) => track_id,
+        None => return MP4PARSE_ERROR_INVALID,
+    };
 
     MP4PARSE_OK
 }
 
 /// Fill the supplied `mp4parse_track_audio_info` with metadata for `track`.
 #[no_mangle]
-pub unsafe extern fn mp4parse_get_track_audio_info(parser: *mut mp4parse_parser, track: u32, info: *mut mp4parse_track_audio_info) -> mp4parse_error {
+pub unsafe extern fn mp4parse_get_track_audio_info(parser: *mut mp4parse_parser, track_index: u32, info: *mut mp4parse_track_audio_info) -> mp4parse_error {
     if parser.is_null() || info.is_null() || (*parser).poisoned() {
         return MP4PARSE_ERROR_BADARG;
     }
 
     let context = (*parser).context_mut();
 
-    if track as usize >= context.tracks.len() {
+    if track_index as usize >= context.tracks.len() {
         return MP4PARSE_ERROR_BADARG;
     }
 
-    let track = &context.tracks[track as usize];
+    let track = &context.tracks[track_index as usize];
 
     match track.track_type {
         TrackType::Audio => {}
@@ -312,23 +383,52 @@ pub unsafe extern fn mp4parse_get_track_audio_info(parser: *mut mp4parse_parser,
     (*info).bit_depth = audio.samplesize;
     (*info).sample_rate = audio.samplerate >> 16; // 16.16 fixed point
 
+    match audio.codec_specific {
+        AudioCodecSpecific::ES_Descriptor(ref v) => {
+            if v.len() > std::u32::MAX as usize {
+                return MP4PARSE_ERROR_INVALID;
+            }
+            (*info).codec_specific_config.length = v.len() as u32;
+            (*info).codec_specific_config.data = v.as_ptr();
+        }
+        AudioCodecSpecific::OpusSpecificBox(ref opus) => {
+            let mut v = Vec::new();
+            match serialize_opus_header(opus, &mut v) {
+                Err(_) => {
+                    return MP4PARSE_ERROR_INVALID;
+                }
+                Ok(_) => {
+                    let header = (*parser).opus_header_mut();
+                    header.insert(track_index, v);
+                    match header.get(&track_index) {
+                        None => {}
+                        Some(v) => {
+                            (*info).codec_specific_config.length = v.len() as u32;
+                            (*info).codec_specific_config.data = v.as_ptr();
+                        }
+                    }
+                }
+            }
+        }
+    }
+
     MP4PARSE_OK
 }
 
 /// Fill the supplied `mp4parse_track_video_info` with metadata for `track`.
 #[no_mangle]
-pub unsafe extern fn mp4parse_get_track_video_info(parser: *mut mp4parse_parser, track: u32, info: *mut mp4parse_track_video_info) -> mp4parse_error {
+pub unsafe extern fn mp4parse_get_track_video_info(parser: *mut mp4parse_parser, track_index: u32, info: *mut mp4parse_track_video_info) -> mp4parse_error {
     if parser.is_null() || info.is_null() || (*parser).poisoned() {
         return MP4PARSE_ERROR_BADARG;
     }
 
     let context = (*parser).context_mut();
 
-    if track as usize >= context.tracks.len() {
+    if track_index as usize >= context.tracks.len() {
         return MP4PARSE_ERROR_BADARG;
     }
 
-    let track = &context.tracks[track as usize];
+    let track = &context.tracks[track_index as usize];
 
     match track.track_type {
         TrackType::Video => {}
@@ -381,8 +481,10 @@ extern fn valid_read(buf: *mut u8, size: usize, userdata: *mut std::os::raw::c_v
 #[test]
 fn new_parser() {
     let mut dummy_value: u32 = 42;
-    let io = mp4parse_io { read: panic_read,
-                           userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void };
+    let io = mp4parse_io {
+        read: panic_read,
+        userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void,
+    };
     unsafe {
         let parser = mp4parse_new(&io);
         assert!(!parser.is_null());
@@ -399,10 +501,13 @@ fn free_null_parser() {
 }
 
 #[test]
-#[should_panic(expected = "assertion failed")]
 fn get_track_count_null_parser() {
     unsafe {
-        let _ = mp4parse_get_track_count(std::ptr::null());
+        let mut count: u32 = 0;
+        let rv = mp4parse_get_track_count(std::ptr::null(), std::ptr::null_mut());
+        assert!(rv == MP4PARSE_ERROR_BADARG);
+        let rv = mp4parse_get_track_count(std::ptr::null(), &mut count);
+        assert!(rv == MP4PARSE_ERROR_BADARG);
     }
 }
 
@@ -427,29 +532,34 @@ fn arg_validation() {
         assert!(parser.is_null());
 
         let mut dummy_value = 42;
-        let io = mp4parse_io { read: std::mem::transmute(null_mut),
-                               userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void };
+        let io = mp4parse_io {
+            read: std::mem::transmute(null_mut),
+            userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void,
+        };
         let parser = mp4parse_new(&io);
         assert!(parser.is_null());
 
         // Passing a null mp4parse_parser is an error.
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_read(std::ptr::null_mut()));
 
-        let mut dummy_info = mp4parse_track_info { track_type: MP4PARSE_TRACK_TYPE_VIDEO,
-                                                   track_id: 0,
-                                                   duration: 0,
-                                                   media_time: 0 };
+        let mut dummy_info = mp4parse_track_info {
+            track_type: MP4PARSE_TRACK_TYPE_VIDEO,
+            codec: mp4parse_codec::MP4PARSE_CODEC_UNKNOWN,
+            track_id: 0,
+            duration: 0,
+            media_time: 0,
+        };
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_info(std::ptr::null_mut(), 0, &mut dummy_info));
 
-        let mut dummy_video = mp4parse_track_video_info { display_width: 0,
-                                                          display_height: 0,
-                                                          image_width: 0,
-                                                          image_height: 0 };
+        let mut dummy_video = mp4parse_track_video_info {
+            display_width: 0,
+            display_height: 0,
+            image_width: 0,
+            image_height: 0,
+        };
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_video_info(std::ptr::null_mut(), 0, &mut dummy_video));
 
-        let mut dummy_audio = mp4parse_track_audio_info { channels: 0,
-                                                          bit_depth: 0,
-                                                          sample_rate: 0 };
+        let mut dummy_audio = Default::default();
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_audio_info(std::ptr::null_mut(), 0, &mut dummy_audio));
     }
 }
@@ -458,8 +568,10 @@ fn arg_validation() {
 fn arg_validation_with_parser() {
     unsafe {
         let mut dummy_value = 42;
-        let io = mp4parse_io { read: error_read,
-                               userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void };
+        let io = mp4parse_io {
+            read: error_read,
+            userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void,
+        };
         let parser = mp4parse_new(&io);
         assert!(!parser.is_null());
 
@@ -474,21 +586,24 @@ fn arg_validation_with_parser() {
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_video_info(parser, 0, std::ptr::null_mut()));
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_audio_info(parser, 0, std::ptr::null_mut()));
 
-        let mut dummy_info = mp4parse_track_info { track_type: MP4PARSE_TRACK_TYPE_VIDEO,
-                                                   track_id: 0,
-                                                   duration: 0,
-                                                   media_time: 0 };
+        let mut dummy_info = mp4parse_track_info {
+            track_type: MP4PARSE_TRACK_TYPE_VIDEO,
+            codec: mp4parse_codec::MP4PARSE_CODEC_UNKNOWN,
+            track_id: 0,
+            duration: 0,
+            media_time: 0,
+        };
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_info(parser, 0, &mut dummy_info));
 
-        let mut dummy_video = mp4parse_track_video_info { display_width: 0,
-                                                          display_height: 0,
-                                                          image_width: 0,
-                                                          image_height: 0 };
+        let mut dummy_video = mp4parse_track_video_info {
+            display_width: 0,
+            display_height: 0,
+            image_width: 0,
+            image_height: 0,
+        };
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_video_info(parser, 0, &mut dummy_video));
 
-        let mut dummy_audio = mp4parse_track_audio_info { channels: 0,
-                                                          bit_depth: 0,
-                                                          sample_rate: 0 };
+        let mut dummy_audio = Default::default();
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_audio_info(parser, 0, &mut dummy_audio));
 
         mp4parse_free(parser);
@@ -496,19 +611,22 @@ fn arg_validation_with_parser() {
 }
 
 #[test]
-#[should_panic(expected = "assertion failed")]
 fn get_track_count_poisoned_parser() {
     unsafe {
         let mut dummy_value = 42;
-        let io = mp4parse_io { read: error_read,
-                               userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void };
+        let io = mp4parse_io {
+            read: error_read,
+            userdata: &mut dummy_value as *mut _ as *mut std::os::raw::c_void,
+        };
         let parser = mp4parse_new(&io);
         assert!(!parser.is_null());
 
         // Our mp4parse_io read should simply fail with an error.
         assert_eq!(MP4PARSE_ERROR_IO, mp4parse_read(parser));
 
-        let _ = mp4parse_get_track_count(parser);
+        let mut count: u32 = 0;
+        let rv = mp4parse_get_track_count(parser, &mut count);
+        assert!(rv == MP4PARSE_ERROR_BADARG);
     }
 }
 
@@ -523,49 +641,60 @@ fn arg_validation_with_data() {
 
         assert_eq!(MP4PARSE_OK, mp4parse_read(parser));
 
-        assert_eq!(2, mp4parse_get_track_count(parser));
+        let mut count: u32 = 0;
+        assert_eq!(MP4PARSE_OK, mp4parse_get_track_count(parser, &mut count));
+        assert_eq!(2, count);
 
-        let mut info = mp4parse_track_info { track_type: MP4PARSE_TRACK_TYPE_VIDEO,
-                                             track_id: 0,
-                                             duration: 0,
-                                             media_time: 0 };
+        let mut info = mp4parse_track_info {
+            track_type: MP4PARSE_TRACK_TYPE_VIDEO,
+            codec: mp4parse_codec::MP4PARSE_CODEC_UNKNOWN,
+            track_id: 0,
+            duration: 0,
+            media_time: 0,
+        };
         assert_eq!(MP4PARSE_OK, mp4parse_get_track_info(parser, 0, &mut info));
         assert_eq!(info.track_type, MP4PARSE_TRACK_TYPE_VIDEO);
+        assert_eq!(info.codec, mp4parse_codec::MP4PARSE_CODEC_AVC);
         assert_eq!(info.track_id, 1);
         assert_eq!(info.duration, 40000);
         assert_eq!(info.media_time, 0);
 
         assert_eq!(MP4PARSE_OK, mp4parse_get_track_info(parser, 1, &mut info));
         assert_eq!(info.track_type, MP4PARSE_TRACK_TYPE_AUDIO);
+        assert_eq!(info.codec, mp4parse_codec::MP4PARSE_CODEC_AAC);
         assert_eq!(info.track_id, 2);
         assert_eq!(info.duration, 61333);
         assert_eq!(info.media_time, 21333);
 
-        let mut video = mp4parse_track_video_info { display_width: 0,
-                                                    display_height: 0,
-                                                    image_width: 0,
-                                                    image_height: 0 };
+        let mut video = mp4parse_track_video_info {
+            display_width: 0,
+            display_height: 0,
+            image_width: 0,
+            image_height: 0,
+        };
         assert_eq!(MP4PARSE_OK, mp4parse_get_track_video_info(parser, 0, &mut video));
         assert_eq!(video.display_width, 320);
         assert_eq!(video.display_height, 240);
         assert_eq!(video.image_width, 320);
         assert_eq!(video.image_height, 240);
 
-        let mut audio = mp4parse_track_audio_info { channels: 0,
-                                                    bit_depth: 0,
-                                                    sample_rate: 0 };
+        let mut audio = Default::default();
         assert_eq!(MP4PARSE_OK, mp4parse_get_track_audio_info(parser, 1, &mut audio));
         assert_eq!(audio.channels, 2);
         assert_eq!(audio.bit_depth, 16);
         assert_eq!(audio.sample_rate, 48000);
 
         // Test with an invalid track number.
-        let mut info = mp4parse_track_info { track_type: MP4PARSE_TRACK_TYPE_VIDEO,
-                                             track_id: 0,
-                                             duration: 0,
-                                             media_time: 0 };
+        let mut info = mp4parse_track_info {
+            track_type: MP4PARSE_TRACK_TYPE_VIDEO,
+            codec: mp4parse_codec::MP4PARSE_CODEC_UNKNOWN,
+            track_id: 0,
+            duration: 0,
+            media_time: 0,
+        };
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_info(parser, 3, &mut info));
         assert_eq!(info.track_type, MP4PARSE_TRACK_TYPE_VIDEO);
+        assert_eq!(info.codec, mp4parse_codec::MP4PARSE_CODEC_UNKNOWN);
         assert_eq!(info.track_id, 0);
         assert_eq!(info.duration, 0);
         assert_eq!(info.media_time, 0);
@@ -580,9 +709,7 @@ fn arg_validation_with_data() {
         assert_eq!(video.image_width, 0);
         assert_eq!(video.image_height, 0);
 
-        let mut audio = mp4parse_track_audio_info { channels: 0,
-                                                    bit_depth: 0,
-                                                    sample_rate: 0 };
+        let mut audio = Default::default();
         assert_eq!(MP4PARSE_ERROR_BADARG, mp4parse_get_track_audio_info(parser, 3, &mut audio));
         assert_eq!(audio.channels, 0);
         assert_eq!(audio.bit_depth, 0);
